﻿--[[
SimpleMD
Author: Michael Joseph Murray aka Starinnia of Lothar(US) aka Lyte of Lothar(US)
$Revision: 96 $
$Date: 2008-10-13 14:51:18 +0000 (Mon, 13 Oct 2008) $
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2007-2008 Michael J. Murray aka Starinnia of Lothar

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.

]]
local ver, rev = "3.0", tonumber(("$Revision: 96 $"):match("%d+"))
local pname = UnitName("player")
local class = select(2, UnitClass("player"))
local addonID = "SimpleMD-" .. ver .. " r" .. rev 

SimpleMD = LibStub("AceAddon-3.0"):NewAddon("SimpleMD", "AceConsole-3.0", "AceComm-3.0", "AceTimer-3.0", "AceEvent-3.0", "LibSink-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SimpleMD")
SimpleMD.fonts = {}
SimpleMD.textures = {}

function SimpleMD:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SimplemdDB", self:GetDefaults(), "Default")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	self:SetSinkStorage(self.db.profile.sinkOpts)
	
	self.anchor, self.frame = self:CreateAnchor(addonID)
	self.anchor.timers = {}
	self.timercount = 0
	self.recyclePile = {}
	self.idents = {}
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SimpleMD", self:GetOptions())
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SimpleMD", "SimpleMD")
	self:RegisterChatCommand("smd", "OpenConfig", true, true)
	self:RegisterChatCommand("simplemd", "OpenConfig")
	LSM.RegisterCallback(self, "LibSharedMedia_Registered")
end

function SimpleMD:OnEnable()
	self:LibSharedMedia_Registered()
	self.tex = [[Interface\Icons\Ability_Hunter_Misdirection]]
	self:RegisterComm("SimpleMD", "OnCommReceive")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "RaidStatus") --to clear static bars after leaving a raid
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
end

function SimpleMD:LibSharedMedia_Registered()
	for k, v in pairs(LSM:List("font")) do
		self.fonts[v] = v
	end
	for k, v in pairs(LSM:List("statusbar")) do
		self.textures[v] = v
	end
end

function SimpleMD:Reset()
	self.anchor:ClearAllPoints()
	self.frame:ClearAllPoints()
	self.anchor:SetPoint("CENTER", UIParent, "CENTER")
	self.frame:SetPoint("CENTER", UIParent, "CENTER")
end

function SimpleMD:Refresh()
	local db = self.db.profile
	self.anchor:ClearAllPoints()
	self.frame:ClearAllPoints()
	if db.position.x then
		self.anchor:SetPoint(db.position.point, UIParent, db.position.anchor, db.position.x, db.position.y)
		self.frame:SetPoint(db.position.point, UIParent, db.position.anchor, db.position.x, db.position.y)
	else
		self.anchor:SetPoint("CENTER", UIParent, "CENTER")
		self.frame:SetPoint("CENTER", UIParent, "CENTER")
	end
end

function SimpleMD:OpenConfig()
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("SimpleMD", 500, 550)
	LibStub("AceConfigDialog-3.0"):Open("SimpleMD")
end

function SimpleMD:OnCommReceive(prefix, message, distro, sender)
	if sender ~= pname then
		if message == "SMDREADINESS" then
			if self.db.profile.showReadiness then
				self:PrintMessage(string.format(self.db.profile.readiness, sender), 2)
			end
			if self.db.profile.showBars then
				self:ReadinessCheck(sender)
			end
		elseif message == "SMDTEN" and self.db.profile.ten then
			self:PrintMessage(string.format(self.db.profile.tenCD, sender), 2)
		elseif message == "SMDREADY" and self.db.profile.ready then
			self:PrintMessage(string.format(self.db.profile.noCD, sender), 2)
		end
	end
end

function SimpleMD:PrintMessage(message, isPlayer)
	--if isPlayer is true the message regards the player and the message needs to
	--be sent to any selected channels, if it is false then the message is about another hunter
	--and should only be displayed locally to the player
	
	if isPlayer then
		--print to chat frame
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		--print to Error Message frame
		elseif self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		--print to Party chat
		elseif self.db.profile.showParty and GetNumRaidMembers() == 0 then
			SendChatMessage(message, "PARTY")
		--print to Raid Chat
		elseif self.db.profile.showRaid and UnitInRaid("player") then
			SendChatMessage(message, "RAID")
		--print to Raid Warning
		elseif (IsRaidLeader() or IsRaidOfficer()) and self.db.profile.showRW then
			SendChatMessage(message, "RAID_WARNING")
		-- print to custom channel
		elseif self.db.profile.showInChannel and self.db.profile.channel ~= "" then
			SendChatMessage(message, "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
		--print to Scrolling Text
		elseif self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
	else
		--print to Scrolling Text mod
		if self.db.profile.showFloating then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		--print to chat frame
		elseif self.db.profile.showInChat then
			self:Print(message)
		--print to error frame
		elseif self.db.profile.showOnScreen then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
	end
end

-- 34477 misdirect on hunter
-- 35079 misdirect gained by target
function SimpleMD:CombatLogEvent(event, timestamp, subevent, sID, sName, sFlags, dID, dName, dFlags, spellID, spellName)
	if spellID == 34477 then
		if subevent == "SPELL_CAST_SUCCESS" then
			if self.db.profile.showBars then
				self:CreateTimerBar(sName, dName, 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp)
			end
			if self.db.profile.showGain then
				self:PrintMessage(string.format(L["%s cast Misdirection on %s"], sName, dName), sName == pname)
			end
			if sName == pname then --if player start CD timers for broadcast
				if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
					self:ScheduleTimer("CooldownBroadcast", 20, 10) -- 10 second warning
					self:ScheduleTimer("CooldownBroadcast", 30, 0) -- cooldown finished
				end
			end
		elseif subevent == "SPELL_AURA_REMOVED" then
			if self.db.profile.showFade then
				self:PrintMessage(string.format(L["Misdirection fades from %s"], dName), sName == pname)
			end
			--if CD bars are being used, change the color to the normal choice from the special signifying active charges
			if self.db.profile.showBars then
				for k, v in pairs(self.idents) do
					if v == dName then
						self.anchor.timers[k].timer:SetStatusBarColor(self.db.profile.barR, self.db.profile.barG, self.db.profile.barB, 1)
					end
				end
			end
		end
	elseif spellID == 23989 then
		if subevent == "SPELL_CAST_SUCCESS" then
			if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 and mdOnCD then
				self:CancelAllTimers() --cancel all CD message events
				if self.db.profile.showReadiness then
					self:PrintMessage(string.format(self.db.profile.readiness, pname), true)
				end
				if UnitInRaid("player") then
					self:SendCommMessage("SimpleMD", "SMDREADINESS", "RAID", nil, "NORMAL")
				else
					self:SendCommMessage("SimpleMD", "SMDREADINESS", "PARTY", nil, "NORMAL")
				end
				if self.db.profile.showSelf and self.db.profile.showBars then
					self:ReadinessCheck(pname)
				end
			end
		end
	end
end

--Function for broadcasting the cooldown messages
--Do nothing if player is dead or ghost since they can't MD anymore (thanks Cerqua)
function SimpleMD:CooldownBroadcast(timeleft)
	if not UnitIsDeadOrGhost("player") and (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) then
		if timeleft == 10 then
			if self.db.profile.ten then
				self:PrintMessage(string.format(self.db.profile.tenCD, pname), true)
			end
			if UnitInRaid("player") then
				self:SendCommMessage("SimpleMD", "SMDTEN", "RAID", nil, "NORMAL")
			else
				self:SendCommMessage("SimpleMD", "SMDTEN", "PARTY", nil, "NORMAL")
			end
			return
		elseif timeleft == 0 then
			if self.db.profile.ready then
				self:PrintMessage(string.format(self.db.profile.noCD, pname), true)
			end
			if UnitInRaid("player") then
				self:SendCommMessage("SimpleMD", "SMDREADY", "RAID", nil, "NORMAL")
			else
				self:SendCommMessage("SimpleMD", "SMDREADY", "PARTY", nil, "NORMAL")
			end
			return
		end
	end
end

--see if the player is in a raid, if not.. clear the CD bars
function SimpleMD:RaidStatus()
	if not UnitInRaid("player") then
		self:ClearTimerBars()
	end
end

--Disable the bars and immediately cancel all running CD timers
function SimpleMD:DisableBars()
	if self.db.profile.showBars then
		self.db.profile.showBars = false
	else
		self.db.profile.showBars = true
	end
	
	self:ClearTimerBars()
end

--Test function for all aspects of the mod
function SimpleMD:Test()
	if self.db.profile.showInChat then
		self:Print(L["Testing Chat Frame Printing"])
	elseif self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(L["Testing On Screen Printing"], self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	elseif self.db.profile.showInChannel and channel ~= "" then
		SendChatMessage(L["Testing Channel Printing"], "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
	elseif self.db.profile.showFloating then
		self:Pour(L["Testing Floating Message"], self.db.profile.r, self.db.profile.g, self.db.profile.b)
	elseif self.db.profile.showBars then
		self:ClearTimerBars()
		self:CreateTimerBar("Test1 ", "test_target", 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp)
		self:CreateTimerBar("test2 ", nil, 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp)
		self:ScheduleTimer("ClearTimerBars", 30)
	end
end
